<?php

namespace Drp\JsonApi\Auth;

use Drp\JsonApi\Exceptions\InvalidOAuthUser;
use Drp\JsonApi\Exceptions\MissingAuthFieldNames;
use Drp\JsonApi\Exceptions\MissingUserProvider;
use Illuminate\Auth\AuthManager;
use Illuminate\Hashing\HashManager;
use Laravel\Passport\Bridge\UserRepository;
use League\OAuth2\Server\Entities\ClientEntityInterface;
use League\OAuth2\Server\Entities\UserEntityInterface;
use League\OAuth2\Server\Exception\OAuthServerException;

class PassportAuthRepository extends UserRepository
{
    /**
     * @var \Illuminate\Auth\AuthManager
     */
    private $authManager;

    public function __construct(AuthManager $authManager, HashManager $hasher)
    {
        $this->authManager = $authManager;
        parent::__construct($hasher);
    }

    /**
     * Get a user entity.
     *
     * @param string $username
     * @param string $password
     * @param string $grantType The grant type used
     * @param \League\OAuth2\Server\Entities\ClientEntityInterface $clientEntity
     *
     * @return \League\OAuth2\Server\Entities\UserEntityInterface
     * @throws \Drp\JsonApi\Exceptions\InvalidOAuthUser
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * phpcs:disable Squiz.Commenting.FunctionComment.ScalarTypeHintMissing -- Passport interface
     */
    public function getUserEntityByUserCredentials(
        $username,
        $password,
        $grantType,
        ClientEntityInterface $clientEntity
    ) {
        $userProvider = $this->getUserProvider();
        $credentials = $this->formatCredentials($username, $password);
        $user = $userProvider->retrieveByCredentials($credentials);

        throw_if($user === null, OAuthServerException::accessDenied('Invalid username or password'));

        if ($user instanceof UserEntityInterface) {
            return $user;
        }

        throw new InvalidOAuthUser($user);
    }
    // phpcs:enable

    /**
     * @return \Illuminate\Contracts\Auth\UserProvider
     * @throws \Drp\JsonApi\Exceptions\MissingUserProvider
     */
    private function getUserProvider()
    {
        $providerName = $this->getCurrentProviderName();
        $userProvider = $this->authManager->createUserProvider($providerName);

        if ($userProvider === null) {
            throw new MissingUserProvider($providerName);
        }

        return $userProvider;
    }

    private function formatCredentials(string $username, string $password): array
    {
        $provider = $this->getCurrentProviderName();
        $credentialFieldNames = config(
            'api.auth_field_names.' . $provider,
            config('api.auth_field_names.default')
        );

        if ($credentialFieldNames === null) {
            throw new MissingAuthFieldNames($provider);
        }

        return array_combine($credentialFieldNames, [$username, $password]);
    }

    /**
     * @return string
     */
    private function getCurrentProviderName(): ?string
    {
        $driver = $this->authManager->getDefaultDriver();

        return config('auth.guards.' . $driver . '.provider');
    }
}
