<?php

namespace Drp\JsonApi\Http\Concerns;

use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Collection;

trait RespondsWithJsonApi
{
    private function isApiCall(Request $request): bool
    {
        return str_contains($request->url(), config('api.url'));
    }

    private function respondWithEmpty(): Response
    {
        return response('', Response::HTTP_NO_CONTENT);
    }

    private function respondWithError(int $status, string $title, string $detail): JsonResponse
    {
        return response()
            ->json([
                'errors' => [
                    'status' => (string) $status,
                    'title' => $title,
                    'detail' => $detail,
                ],
            ])
            ->setStatusCode($status)
            ->setEncodingOptions(JSON_UNESCAPED_SLASHES);
    }

    private function respondWithValidationError(array $errors): JsonResponse
    {
        return response()
            ->json([
                'errors' => $this->generateErrors($errors),
            ])
            ->setStatusCode(Response::HTTP_UNPROCESSABLE_ENTITY)
            ->setEncodingOptions(JSON_UNESCAPED_SLASHES);
    }

    /**
     * Build an array of error objects for each type of error given.
     *
     * Structure needs to be:
     *
     * 'resource-type' => [
     *   'attribute' => [
     *     'error message'
     *   ]
     * ]
     *
     * if you have an id related to the resource then add a suffix of the id to the type
     * e.g.
     *
     * 'resource-type_1234' => [
     *   'attribute' => [
     *     'error message'
     *   ]
     * ]
     */
    private function generateErrors(array $errors): Collection
    {
        return collect($errors)->map(function ($item, $type) {
            return collect($item)->flatMap(function ($detail, $attribute) use ($type) {
                $detail = array_wrap($detail);

                return array_map(function ($detail) use ($type, $attribute) {
                    $source = $this->buildInitialSource($type);

                    if (is_numeric($attribute) === false) {
                        $source['attribute'] = $attribute;
                    }

                    return [
                        'source' => $source,
                        'status' => (string) Response::HTTP_UNPROCESSABLE_ENTITY,
                        'detail' => $detail,
                    ];
                }, $detail);
            });
        })
        ->values()
        ->flatten(1);
    }

    private function buildInitialSource(string $type): array
    {
        $parts = explode('_', $type);

        if (count($parts) === 1) {
            return [
                'type' => $type,
            ];
        }

        return [
            'type' => $parts[0],
            'id' => $parts[1]
        ];
    }
}
